/** @file SL2_Z.cpp
 * The groups GL2(Z) and SL2(Z) (optionally with stored ST-decomposition).
 * The algorithm which computes an ST-decomposition for any element of
 * SL(2,Z) can be found for example in [1], Section 3.1.
 *
 * [1] G. Schmithuesen : "An Algorithm for Finding the Veech Group of an Origami"
 *     Experiment. Math. Volume 13, Issue 4 (2004), 459-472.
 * 
 * @author Karsten Kremer
 * @version 1.0
 * @date 29.01.2007
 */

/*  This file is part of the Origami-Library, which is a collection
 *  of algorithms that calculate properties of mathematical objects
 *  called "origamis".
 *
 *  Copyright (C) 2007 Karsten Kremer
 *  Address: Institute for Algebra and Geometry, Fakultaet fuer
 *  Mathematik, Universitaet Karlsruhe, D-76128 Karlsruhe, Germany.
 *  E-mail: karsten.kremer@math.uni-karlsruhe.de
 *  or origami@math.uni-karlsruhe.de
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 3
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, see
 *  <http://www.gnu.org/licenses/> or write to the Free
 *  Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 *  Boston, MA  02110-1301, USA.
 *
 *  If you publish results you obtained by using this software please
 *  don't forget to give appropriate credit in your publication.
 */



#include <cstdio>
#include <cassert>
#include "matrix.h"
#include "freeGroups.h"
#include "SL2_Z.h"

using namespace std;

// **********************+ GL2_Z
		
void GL2_Z::set(int aa, int bb, int cc, int dd) {
	a = aa; b = bb; c = cc; d = dd;
	assert (isInvertible());
}

// **********************+ SL2_Z

void SL2_Z::set(int aa, int bb, int cc, int dd) {
	a = aa; b = bb; c = cc; d = dd;
	assert (det()==1);
}

void SL2_Z::setST (const F2& st) { 
	// ST-Zerlegung (0 = S = (0,-1; 1,0), (1 = T = (1,1; 0,1)
	F2Map<SL2_Z> f(SL2_Z(0,-1,1,0),SL2_Z(1,1,0,1));
	setMatrix(f.image(st));
}

F2   SL2_Z::getST () const {		  
	// ST-Zerlegung (0 = S = (0,-1; 1,0), 1 = T = (1,1; 0,1)
	//matrixwort zerlege(matrix AA)
	int i = 1;
	F2 result('S','T'); 
	SL2_Z M = *this; 
	F2 S(0), T(1);
	int k,r,a,b,c,d;
	while(M.getC() != 0 && i < 10) {
		i++;
		a = M.getA(); 
		b = M.getB();
		c = M.getC(); 
		d = M.getD();
		k = d/c; r = d%c; 
		M = SL2_Z(-a*k+b,-a,r,-c);
		result.mult_left(T^k);
		result.mult_left(S.inverse());
	};
	a = M.getA(); 
	b = M.getB();
	if (a == -1) {
		result.mult_left(S^2);
		b = b * (-1);
	}
	result.mult_left(T^b);
	return result; 
}

AutF2 SL2_Z::STtoAutF2 (const F2& st) {
	F2 x(0), y(1);
	AutF2 gS(EndF2(y,x^-1), EndF2(y^-1,x));
	AutF2 gT(EndF2(x,x*y),  EndF2(x,(x^-1)*y));
	gS.invert();
	F2Map<AutF2> g(gS,gT);
	return g.image(st);
}
