/** @file SL2_Z.h
 * The groups GL2(Z) and SL2(Z) (optionally with stored ST-decomposition).
 * The algorithm which computes an ST-decomposition for any element of
 * SL(2,Z) can be found for example in [1], Section 3.1.
 *
 * [1] G. Schmithuesen : "An Algorithm for Finding the Veech Group of an Origami"
 *     Experiment. Math. Volume 13, Issue 4 (2004), 459-472.
 * 
 * @author Karsten Kremer
 * @version 1.0
 * @date 29.01.2007
 */

/*  This file is part of the Origami-Library, which is a collection
 *  of algorithms that calculate properties of mathematical objects
 *  called "origamis".
 *
 *  Copyright (C) 2007 Karsten Kremer
 *  Address: Institute for Algebra and Geometry, Fakultaet fuer
 *  Mathematik, Universitaet Karlsruhe, D-76128 Karlsruhe, Germany.
 *  E-mail: karsten.kremer@math.uni-karlsruhe.de
 *  or origami@math.uni-karlsruhe.de
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 3
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, see
 *  <http://www.gnu.org/licenses/> or write to the Free
 *  Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 *  Boston, MA  02110-1301, USA.
 *
 *  If you publish results you obtained by using this software please
 *  don't forget to give appropriate credit in your publication.
 */
 
#if !defined(ORIGAMI_SL2_Z_H_INCLUDED)
#define ORIGAMI_SL2_Z_H_INCLUDED

#include <iostream>
#include <cstdio>
#include <cassert>
#include "matrix.h"

using namespace std;

class AutF2;
class F2;

class GL2_Z : public IntMatrix2x2 {
	public:
		GL2_Z(int aa, int bb, int cc, int dd)				{set(aa,bb,cc,dd);}
		GL2_Z()														{set(1,0,0,1);} 
		GL2_Z(const IntMatrix2x2& other) 					{setMatrix(other);}
		
		void set(int aa, int bb, int cc, int dd);
		GL2_Z copy()   								 const	{return GL2_Z(*this);}
		
		GL2_Z operator * (const GL2_Z& other) const 	{return copy().mult(other);}
		GL2_Z operator / (const GL2_Z& other) const 	{return copy().div(other);}
		GL2_Z operator ^ (int n) 				 const 	{return copy().power(n);}
		GL2_Z operator - () 						 const 	{return copy().scale(-1);}
		GL2_Z inverse() 								 const 	{return copy().invert();}
		
		void operator = (const IntMatrix2x2& other)  	{setMatrix(other);}
};

class SL2_Z : public IntMatrix2x2 {
	public:
		SL2_Z(int aa, int bb, int cc, int dd)				{set(aa,bb,cc,dd);}
		SL2_Z()														{set(1,0,0,1);} 
		SL2_Z(const IntMatrix2x2& other) 					{setMatrix(other);}
		
		void set(int aa, int bb, int cc, int dd);
		SL2_Z copy()   								 const	{return SL2_Z(*this);}
		
		SL2_Z operator * (const SL2_Z& other) const 	{return copy().mult(other);}
		SL2_Z operator / (const SL2_Z& other) const 	{return copy().div(other);}
		SL2_Z operator ^ (int n) 				 const	{return copy().power(n);}
		SL2_Z operator - () 						 const	{return copy().scale(-1);}
		SL2_Z inverse() 								 const	{return copy().invert();}

		
		void operator = (const IntMatrix2x2& other)  	{setMatrix(other);}
		

		F2   getST () const;				// ST-Zerlegung (0 = S = (0,-1; 1,0)
		void setST (const F2& st);    // 						(1 = T = (1, 1; 0,1)
		static AutF2 STtoAutF2 (const F2& st);		 // Lift in AutF2+
		AutF2 getLift () 									const {return STtoAutF2(getST());}	
};


class SL2_Z_ST {  // Speichert zusätzlich ST-Zerlegung
			private: 
				SL2_Z M;
				F2 ST; 
				SL2_Z_ST(SL2_Z mat, F2 st)						{M = mat; ST = st; ST.setChar('S','T');}
			public:
				SL2_Z_ST()											{ST.setChar('S','T');}
				SL2_Z_ST(int a, int b, int c, int d)		{M.set(a,b,c,d); ST=M.getST();}				
				SL2_Z_ST(F2 st)									{M.setST(st); ST = st; ST.setChar('S','T');}
				SL2_Z getSL2 () 							const	{return M;}
				F2 getST () 								const	{return ST;}
				
				void setOne() 										{M.setOne(); ST.setOne();}
				bool isOne() 								const	{return M.isOne();}
				bool equals(const SL2_Z_ST& other)	const	{return M.equals(other.M);}

				SL2_Z_ST inverse() 						const	{return SL2_Z_ST(M.inverse(), ST.inverse());}
				
				void operator *= (const SL2_Z_ST& other) 			 {M *= other.M; ST *= other.ST;}
				void operator /= (const SL2_Z_ST& other) 			 {M /= other.M; ST /= other.ST;}
				bool operator == (const SL2_Z_ST& other) 	 const {return equals(other);}
				bool operator != (const SL2_Z_ST& other) 	 const {return !equals(other);}
				
				SL2_Z_ST operator * (const SL2_Z_ST& other) const {return SL2_Z_ST(M*other.M, ST*other.ST);}
				SL2_Z_ST operator / (const SL2_Z_ST& other) const {return SL2_Z_ST(M/other.M, ST/other.ST);}
				SL2_Z_ST operator ^ (int n) 					 const {return SL2_Z_ST(M^n, ST^n);}
				SL2_Z_ST operator - () 							 const {return SL2_Z_ST(-M, (F2(0)^2)*ST);}
				void operator = (const SL2_Z_ST& other) 			 {M=other.M; ST=other.ST;}
				
				AutF2 getLift () 										 const {return SL2_Z::STtoAutF2(ST);}
		};

ostream& operator << (ostream& os, const SL2_Z_ST& M) {
	os << M.getSL2() << " = " << M.getST(); 
	return os;
}

#endif // ORIGAMI_SL2_Z_H_INCLUDED
