/** @file graph.h
 * Adjacency graph for an origami.
 * An origami is defined by two permutations. The permutations can be loaded
 * from text-files (with file-extension .ori).
 * 
 * @author Karsten Kremer
 * @version 1.0
 * @date 29.01.2007
 */

/*  This file is part of the Origami-Library, which is a collection
 *  of algorithms that calculate properties of mathematical objects
 *  called "origamis".
 *
 *  Copyright (C) 2007 Karsten Kremer
 *  Address: Institute for Algebra and Geometry, Fakultaet fuer
 *  Mathematik, Universitaet Karlsruhe, D-76128 Karlsruhe, Germany.
 *  E-mail: karsten.kremer@math.uni-karlsruhe.de
 *  or origami@math.uni-karlsruhe.de
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 3
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, see
 *  <http://www.gnu.org/licenses/> or write to the Free
 *  Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 *  Boston, MA  02110-1301, USA.
 *
 *  If you publish results you obtained by using this software please
 *  don't forget to give appropriate credit in your publication.
 */
 
#if !defined(ORIGAMI_GRAPH_H_INCLUDED)
#define ORIGAMI_GRAPH_H_INCLUDED

#include <cstdio>
#include <cassert>
#include <iostream>
#include "permutation.h"

using namespace std;

class Vertex;
class F2;
class AutF2;

class Graph {
	friend class Vertex;
	public:
		class iterator {
			public:
				iterator(const Graph& graph, int index) {g = &graph; i = index;} 
				Vertex operator * ();
				void operator ++ (int x) {if (i<=g->getSize()+1) i++;}
				bool operator == (const iterator& other) const {return g==other.g && i==other.i;}
				bool operator != (const iterator& other) const {return g!=other.g || i!=other.i;}
				const Graph* g;
				int i;
		};
		Graph () {size=1;};
		Graph (int n); // n : Anzahl der Ecken
		Graph (char* name); // Öffne Datei
		Graph (int n, const Permutation& x, const Permutation& y); 
		Graph (const Graph& other);
		void resize (int n);
		void set (int n, const Permutation& x, const Permutation& y);
		void save (char* name) const; // Speichere Datei
		bool isVertex (int index) const;
		void addXedge (int a, int b); // b ist rechter Nachbar von a
		void addYedge (int a, int b); // b ist oberer Nachbar von a
		void addXYedges (int a, int b, int c); // b ist rechter, c oberer Nachbar von a
		bool isValid () const; // hat jede Ecke vier Nachbarn?
		const Permutation& getXperm() const {return neighbours[0];}
		const Permutation& getYperm() const {return neighbours[1];}
		Permutation getCommutator() const;
		Signature getSignature() const {return Signature(getCommutator());}
		int getSize() const; // Anzahl der Ecken
		Vertex getVertex (int index) const; 
		int getNumberOfPoints() const; // Anzahl der Punktierungen
		int getGenus() const;
		iterator begin() const;
		iterator end() const;
		int getNeighbour(int vertex, int dir) const;
		int getNeighbour(int vertex, const F2& dir) const;
		void applyAutF2(const AutF2& f, Graph& newGraph) const;
		void applyST(const F2& f, Graph& newGraph) const;
		Graph applyST(const F2& f) const;
		void operator = (const Graph& other);
	private:
		int size;
		Permutation neighbours[4];
};


class Vertex {
	public:
		Vertex () {graph=0;}
		Vertex (const Graph& g, int idx = 1); 
		const Graph* getGraph () const {return graph;}
		void setIndex	(int idx);
		int  getIndex  () const;
		int  getRight  () const;
		int  getLeft   () const;
		int  getTop    () const;
		int  getBottom () const;
		int  getNeighbour (int dir) const; // 0 <= direction <= 3
		int  getNeighbour (const F2& dir) const;
		Vertex getRightV  () const {return graph->getVertex(getRight());}
		Vertex getLeftV   () const {return graph->getVertex(getLeft());}
		Vertex getTopV    () const {return graph->getVertex(getTop());}
		Vertex getBottomV () const {return graph->getVertex(getBottom());}
		Vertex getNeighbourV (int dir) const {return graph->getVertex(getNeighbour(dir));}
		Vertex getNeighbourV (const F2& dir) const {return graph->getVertex(getNeighbour(dir));}

		int getRamificationIndex() const; // of lower left corner

		bool operator == (const Vertex& other) const 
			{return graph==other.graph && index==other.index;}
		bool operator != (const Vertex& other) const 
			{return graph!=other.graph || index!=other.index;}
	private:
		const Graph* graph;
		int index;
	};

ostream& operator << (ostream &os, const Vertex& v); 



ostream& operator << (ostream &os, const Graph& g); 
istream& operator >> (istream &is, Graph& g); 

#endif // ORIGAMI_GRAPH_H_INCLUDED
