/** @file path.h
 * Paths, groups of paths, and spanning trees for origamis.
 * This is used to compute the fundamental group of an origami.
 * 
 * @author Karsten Kremer
 * @version 1.1
 * @date 22.02.2007
 */

/*  This file is part of the Origami-Library, which is a collection
 *  of algorithms that calculate properties of mathematical objects
 *  called "origamis".
 *
 *  Copyright (C) 2007 Karsten Kremer
 *  Address: Institute for Algebra and Geometry, Fakultaet fuer
 *  Mathematik, Universitaet Karlsruhe, D-76128 Karlsruhe, Germany.
 *  E-mail: karsten.kremer@math.uni-karlsruhe.de
 *  or origami@math.uni-karlsruhe.de
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 3
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, see
 *  <http://www.gnu.org/licenses/> or write to the Free
 *  Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 *  Boston, MA  02110-1301, USA.
 *
 *  If you publish results you obtained by using this software please
 *  don't forget to give appropriate credit in your publication.
 */
 
#if !defined(ORIGAMI_PATH_H_INCLUDED)
#define ORIGAMI_PATH_H_INCLUDED

#include <cstdio>
#include <cassert>
#include <iostream>


using namespace std;

class F2;
class Vertex;
class Graph;

class Path {
	public:
		Path (const Vertex& startV);
		void go (int dir);
		void go (const F2& dir);
		void append (const Path& other);
		void flip ();
		const Graph* getGraph() const;
		int getLength() const;
		const F2& getDir() const;
		int getFirstDir() const;
		int getLastDir() const;
		Vertex getFirst() const;
		Vertex getLast() const;
		void pop_front();
		void pop_back();
	private:
		F2 path;
		Vertex start;
		Vertex end;
};
	
ostream& operator << (ostream &os, const Path& p); 

class PathGroup {
	public:
		typedef list<F2>::const_iterator iterator;
		PathGroup(const Vertex& baseV) 	{base = baseV;}
		PathGroup(const Path& p)		 	{base = p.getFirst(); add(p);}
		PathGroup(const PathGroup& other){base = other.base; generators = other.generators;}
		const Graph* getGraph() const 	{return base.getGraph();}
		Vertex getBase() const				{return base;}
		int getSize() const					{return generators.size();}
		void add(const Path& p);
		iterator begin() const				{return generators.begin();}
		iterator end() const					{return generators.end();}		
		const list<F2>& getGenerators() 	{return generators;}
		bool stabilizes (const Vertex& v) const;
		bool stabilizesAll (const Graph& g) const;
		bool stabilizesOne (const Graph& g) const;
		int countFixedPoints (const Graph& g) const;
		F2 simplify(); // Move basepoint to get minimal lengths of the generators
	private:
		Vertex base;
		list<F2> generators;
};

ostream& operator << (ostream &os, const PathGroup& p); 

class SpanningTree {
	public:
		SpanningTree(const Graph& graph);
		SpanningTree(const Vertex& start);
		SpanningTree(const Vertex& start, int* parentDirections);
		~SpanningTree();
		const Graph* getGraph()										const;
		int getSize()													const;
		int getParentDir(const Vertex& v)						const;
		int getParent(const Vertex& v) 							const;
		bool isParent(const Vertex& v, int dir)				const;
		bool contains(const Vertex& v, int dir)				const;
		Vertex getParentV(const Vertex& v) 						const;
		Vertex getRoot()												const;
		bool isRoot(const Vertex& v)								const;
		void goToRoot (Path& path)									const;
		void goToVertex (Path& path, const Vertex& target)	const;	
		Path getPathToRoot (const Vertex& v)					const;
		Path getPath (const Vertex& v, const Vertex& w)		const;	
		PathGroup getFundamentalGroup () 						const; 
		PathGroup getRelations() 									const;
	private:
		Vertex root;
		int* parentDir;
		void findSpanningTree(int start);
};

ostream& operator << (ostream &os, const SpanningTree& tree); 

#endif // ORIGAMI_PATH_H_INCLUDED


