/** @file veechgruppe.h
 * Generators and coset-representatives of the veech-group of an origami.
 * The algorithm is described in [1].
 *
 * [1] G. Schmithuesen : "An Algorithm for Finding the Veech Group of an Origami"
 *     Experiment. Math. Volume 13, Issue 4 (2004), 459-472.

 *
 * @author Karsten Kremer and Gabriela Schmithuesen
 * @version 1.0
 * @date 29.01.2007
 */

/*  This file is part of the Origami-Library, which is a collection
 *  of algorithms that calculate properties of mathematical objects
 *  called "origamis".
 *
 *  Copyright (C) 2007 Karsten Kremer and Gabriela Schmithuesen
 *  Address: Institute for Algebra and Geometry, Fakultaet fuer
 *  Mathematik, Universitaet Karlsruhe, D-76128 Karlsruhe, Germany.
 *  E-mail: karsten.kremer@math.uni-karlsruhe.de, 
 *  schmithuesen@mathematik.uni-karlsruhe.de
 *  or origami@math.uni-karlsruhe.de
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 3
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, see
 *  <http://www.gnu.org/licenses/> or write to the Free
 *  Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 *  Boston, MA  02110-1301, USA.
 *
 *  If you publish results you obtained by using this software please
 *  don't forget to give appropriate credit in your publication.
 */
 
#if !defined(ORIGAMI_VEECHGRUPPE_H_INCLUDED)
#define ORIGAMI_VEECHGRUPPE_H_INCLUDED

#include <iostream>
#include <cmath>
#include <list>
#include <set>
#include <vector>
#include <algorithm>
#include <functional>

#include "graph.h"
#include "freeGroups.h"
#include "path.h"
#include "SL2_Z.h"
#include "triangulierung.h"

class Veechgruppe {
	public:
		list<SL2_Z_ST> affine_gruppe;
		triang origami;
		bool abbruch;
		
		Veechgruppe(const Graph& origami, int stop = 0);
		~Veechgruppe() {delete pi; delete buf;}
	private:
		class pDreieck {
			public:
				pDreieck(dreieck* q) {p = q; size = p->return_inhalt().getST().getLength();};
				dreieck* p;
				int size;
				bool operator < (const pDreieck& other) const {return size < other.size;}
		};
		multiset<pDreieck> todo; //all triangles whose S-edges is not glued to any other edge
		const Graph* g;	  // Origami
		Graph* buf; 

		PathGroup* pi; // fundamental group
		bool affin_pruefen (const SL2_Z_ST& A);
		void neues_S_dreieck (dreieck* d);
		void neue_T_dreiecke (dreieck* d);
		void vs_st	(int stop);
};

Veechgruppe::Veechgruppe(const Graph& origami, int stop) 	
	{
		g = &origami;
		buf = new Graph(g->getSize());
		SpanningTree t(*g);
		pi = new PathGroup(t.getFundamentalGroup());
		vs_st(stop);
	}		



bool Veechgruppe::affin_pruefen (const SL2_Z_ST& A) { 
	g->applyST(A.getST(),*buf); 
	return pi->stabilizesOne(*buf);
}


void Veechgruppe::neues_S_dreieck (dreieck* d) {
	SL2_Z_ST B,E;
	bool in_affiner_gruppe;
	multiset<pDreieck>::iterator it;
					
	B = d->return_inhalt() * SL2_Z_ST(F2(0)); // S*A or T*A, depends on translattyp

    	//check whether B is already icluded in the affine group:
	
	in_affiner_gruppe = false;
	for (it = todo.begin(); !in_affiner_gruppe && it != todo.end(); it++) {
		E = B * it->p->return_inhalt().inverse();

      	//check, whether B and D are in the same coset:
		if (affin_pruefen(E)||affin_pruefen(-E)) {
       		//if E is in the affine group, then add it to the list:
			in_affiner_gruppe = true;
			affine_gruppe.push_back(E);
         	// the two triangles A and D are glued:

			origami.verkleben(0,d,it->p);
			// all edges of D are glued to another edge
			if (d != it->p) todo.erase(it); // d will later be deleted from vs_st 
		}
	}
    	//if not, add the triangle ("dreieck"):
	if (!in_affiner_gruppe) neue_T_dreiecke (origami.anfuegen(0, dreieck(B), d));
}


void Veechgruppe::neue_T_dreiecke (dreieck* d) {
	SL2_Z_ST A,B,E,T(F2(1));
	dreieck* d2 = d;
	bool in_affiner_gruppe = false;
	A = d->return_inhalt(); 
	B=A;
	while (!in_affiner_gruppe) {
		B=B*T; 
		E=B*A.inverse();
	    	//while E is not included in the affine group:
		if (affin_pruefen(E) || affin_pruefen(-E)) in_affiner_gruppe = true;
		else {
			d2 = origami.anfuegen(1, dreieck(B), d2);
			todo.insert(pDreieck(d2));
			A=A*T.inverse(); 
			E=B*A.inverse();
			if (affin_pruefen(E) || affin_pruefen(-E)) in_affiner_gruppe = true;
			else {
				d = origami.anfuegen(3, dreieck(A), d);
				todo.insert(pDreieck(d));
			}
		}
	}
  //now: E is in the affine group:
	affine_gruppe.push_back(E);
// the last triangle d2 is glued to the first triangle d
	origami.verkleben(1,d2,d);	
}

// vs_st determines system of coset representatives  and generators of the affine group 
// It calculates the genus and the number of cusps of the origami curve
void Veechgruppe::vs_st (int stop) {

	dreieck* d;
	multiset<pDreieck>::iterator it;

 	// Anfangselment ist Einheitsmatrix
	abbruch = false;
	todo.clear();
	origami = triang(dreieck());
	
	d = &*origami.begin();
	neue_T_dreiecke(d);
	todo.insert(pDreieck(d));
	while (!todo.empty() && !abbruch) {
		it = todo.begin();
		d = it->p; 
		neues_S_dreieck(d);
		todo.erase(it);
		if (stop > 0 && origami.return_flaechenzahl()>=stop) abbruch = true;		
	}
   
	origami.ecken_bestimmen();

}


#endif /* ORIGAMI_VEECHGRUPPE_H_INCLUDED */

